from PIL import Image,ImageEnhance
import numpy as np
def ReduceOpacity(im, opacity):
    if im.mode != 'RGBA':
        im = im.convert('RGBA')
    else:
        im = im.copy()
    alpha = im.split()[3]
    alpha = ImageEnhance.Brightness(alpha).enhance(opacity)
    im.putalpha(alpha)
    return im
def hideImage(im):
    return ReduceOpacity(im,0)

def create_virgin_layer(w, h):
    return np.asarray(Image.new('RGBA', (w, h), (1, 1, 1, 0)))



import math
from moviepy.editor import VideoFileClip, AudioFileClip
from moviepy.video.fx import make_loopable, loop
from bgeditor.common.utils import get_dir
import uuid
import os
import shutil

def create_suource_can_loop_path(clip_path, is_delete = True, ext="mp4"):
    return create_source_can_loop(get_dir('download'), clip_path, is_delete = is_delete, ext=ext)
def create_source_can_loop(dir_path, clip_path, is_delete = True, ext = "mp4" ):
    tmp_clip_path_z_resync = None
    try:
        if dir_path.endswith("/"):
            dir_path = dir_path[:-1]
        clip = VideoFileClip(clip_path,audio=False)
        tmp_clip_path_z = os.path.join(dir_path, str(uuid.uuid4()) + '-' + os.path.basename(clip_path))
        clip.write_videofile(tmp_clip_path_z, fps=24, codec='libx264')
        clip.close()
        clip = VideoFileClip(tmp_clip_path_z, audio=False)
        cross = clip.duration/5
        if cross > 3:
            cross=3
        clip = make_loopable.make_loopable(clip, cross)
        tmp_clip_path_z = os.path.join(dir_path, str(uuid.uuid4()) + '-' + os.path.basename(clip_path).split(".")[0] + "." + ext)
        tmp_clip_path_z_resync = os.path.join(dir_path , str(uuid.uuid4()) + '-re-sync-' + os.path.basename(clip_path).split(".")[0] + "." + ext)
        try:
            clip.write_videofile(tmp_clip_path_z, fps=24, codec='libx264')
            clip.close()
            tmp_clip_path_z_resync=tmp_clip_path_z
        except:
            cmd = "ffmpeg -y -i \"%s\" -c:v libx264 -crf 22 \"%s\"" % (tmp_clip_path_z, tmp_clip_path_z_resync)
            os.system(cmd)
            os.remove(tmp_clip_path_z)
            pass
        if is_delete:
            os.remove(clip_path)
        try:
            clip = VideoFileClip(tmp_clip_path_z_resync)
            if clip.duration is None or clip.duration < 0.1:
                tmp_clip_path_z_resync = None
        except:
            tmp_clip_path_z_resync = None
            pass
    except:
        pass
    return tmp_clip_path_z_resync

def create_loop_audio_times(audio_path, times, is_del=True):
    try:
        tmp_clip_path = os.path.join(get_dir('coolbg_ffmpeg'), str(uuid.uuid4()) + '-' + os.path.basename(audio_path))
        shutil.copyfile(audio_path, tmp_clip_path)
        file_merg_path = os.path.join(get_dir('coolbg_ffmpeg'), str(uuid.uuid4()))
        final_clip_path = os.path.join(get_dir('coolbg_ffmpeg'), str(uuid.uuid4()) + '-final-' + os.path.basename(audio_path))
        file_merg = open(file_merg_path, "a")
        for i in range(times):
            file_merg.write("file '%s'\n" % tmp_clip_path)
        file_merg.close()
        cmd = "ffmpeg -y -f concat -safe 0 -i \"%s\" -codec copy \"%s\"" % (file_merg_path, final_clip_path)
        os.system(cmd)
        os.remove(tmp_clip_path)
        os.remove(file_merg_path)
        if is_del:
            os.remove(audio_path)
        return final_clip_path
    except:
        pass
    return None
def create_loop_audio(audio_path, loop_duration):
    if loop_duration < 0 : loop_duration = 600
    try:
        audio_clip = AudioFileClip(audio_path)
        clip_duration = audio_clip.duration
        audio_clip.close()
        if clip_duration > loop_duration:
            return audio_path
        tmp_clip_path = os.path.join(get_dir('coolbg_ffmpeg'), str(uuid.uuid4()) + '-' + os.path.basename(audio_path))
        shutil.copyfile(audio_path, tmp_clip_path)
        times = int(math.ceil(loop_duration / clip_duration))
        file_merg_path = os.path.join(get_dir('coolbg_ffmpeg') , str(uuid.uuid4()))
        final_clip_path = os.path.join(get_dir('coolbg_ffmpeg'), str(uuid.uuid4()) + '-final-' + os.path.basename(audio_path))
        file_merg = open(file_merg_path, "a")
        for i in range(times):
            file_merg.write("file '%s'\n" % tmp_clip_path)
        file_merg.close()
        cmd = "ffmpeg -y -f concat -safe 0 -i \"%s\" -codec copy \"%s\"" % (file_merg_path, final_clip_path)
        os.system(cmd)
        os.remove(tmp_clip_path)
        os.remove(file_merg_path)
        clip = AudioFileClip(final_clip_path)
        clip_duration = clip.duration
        clip.close()
        if clip_duration < loop_duration:
            return None
        return final_clip_path
    except:
        pass
    return None
def add_null_sound(input):
    output=  os.path.join(get_dir('coolbg_ffmpeg') ,str(uuid.uuid4()) + '-' + os.path.basename(input))
    cmd = f"ffmpeg -f lavfi -i anullsrc -i \"{input}\" -c:v copy -b:a 128000 -ar 44100 -c:a mp3 -map 0:a -map 1:v -shortest \"{output}\""
    os.system(cmd)
    return output
def merge_list_video(vids, is_del=True):
    file_merg_path = os.path.join(get_dir('coolbg_ffmpeg'), str(uuid.uuid4()))
    final_clip_path = os.path.join(get_dir('results') , str(uuid.uuid4()) + '-final-' + os.path.basename(vids[0]))
    file_merg = open(file_merg_path, "a")
    arrtmp = []
    for vid in vids:
        tmp_clip_path = os.path.join(get_dir('coolbg_ffmpeg'), str(uuid.uuid4()) + '-' + os.path.basename(vid))
        shutil.copyfile(vid, tmp_clip_path)
        if is_del:
            os.remove(vid)
        file_merg.write("file '%s'\n" % tmp_clip_path)
        arrtmp.append(tmp_clip_path)
    file_merg.close()
    cmd = "ffmpeg -y -f concat -safe 0 -i \"%s\" -codec copy \"%s\"" % (file_merg_path, final_clip_path)
    os.system(cmd)
    os.remove(file_merg_path)
    for item in arrtmp:
        os.remove(item)
    clip = VideoFileClip(final_clip_path)
    clip_duration = clip.duration
    clip.close()
    if clip_duration < 1:
        return None
    return final_clip_path


def merge_intro_outro(clip_path, intro = None, outro = None):
    arrVids=[]
    if not intro and not outro:
        return clip_path
    if intro:
        arrVids.append(intro)
    arrVids.append(clip_path)
    if outro:
        arrVids.append(outro)
    return merge_list_video(arrVids, True)

def create_video_audio(clip_path, audio_path, rs_path=None, is_del=True):
    try:
        tmp_clip_path_z=rs_path
        if tmp_clip_path_z is None:
            tmp_clip_path_z = os.path.join(get_dir('coolbg_ffmpeg'), str(uuid.uuid4()) + '-final.mp4')
        cmd = f"ffmpeg -i \"{clip_path}\" -i \"{audio_path}\" -vcodec copy -acodec copy -map 0:v -map 1:a -shortest -flags global_header -y \"{tmp_clip_path_z}\""
        os.system(cmd)
        if is_del:
            os.remove(clip_path)
            os.remove(audio_path)
    except:
        tmp_clip_path_z = None
        pass
    return tmp_clip_path_z
def convert_gif(video_path, is_del=True):
    tmp_clip_path_z = os.path.join(get_dir('coolbg_ffmpeg'), str(uuid.uuid4()) + '-gif.avi')
    cmd=f"ffmpeg -i \"{video_path}\"  -c:v ffv1 \"{tmp_clip_path_z}\""
    os.system(cmd)
    if is_del:
        os.remove(video_path)
    return tmp_clip_path_z
def create_loop(clip_path, loop_duration, can_loopable = True):
    try:
        clip = VideoFileClip(clip_path)
        if not can_loopable:
            try:
                if clip.duration > loop_duration:
                    return clip_path
                clip = make_loopable.make_loopable(clip, 0.5)
                tmp_clip_path_z = os.path.join(get_dir('coolbg_ffmpeg'), str(uuid.uuid4()) + '-' + os.path.basename(clip_path))
                #tmp_clip_path_z_resync = get_dir('coolbg_ffmpeg') + str(uuid.uuid4()) + '-re-sync-' + os.path.basename(clip_path).split(".")[0]+".avi"
                clip.write_videofile(tmp_clip_path_z, fps=clip.fps, codec='libx264')
                clip.close()
                # cmd = "ffmpeg -y -i \"%s\" \"%s\"" % (tmp_clip_path_z, tmp_clip_path_z_resync)
                # os.system(cmd)
                # os.remove(tmp_clip_path_z)
                clip = VideoFileClip(tmp_clip_path_z)
                tmp_clip_path = tmp_clip_path_z
            except:
                pass
        else:
            tmp_clip_path = os.path.join(get_dir('coolbg_ffmpeg'), str(uuid.uuid4()) + '-' + os.path.basename(clip_path))
            shutil.copyfile(clip_path, tmp_clip_path)

        clip_duration = clip.duration
        clip.close()
        if clip_duration > loop_duration:
            return clip_path
        times = int(math.ceil(loop_duration /clip_duration))+1
        # print(times)
        file_merg_path = os.path.join(get_dir('coolbg_ffmpeg'), str(uuid.uuid4()))
        final_clip_path = os.path.join(get_dir('coolbg_ffmpeg'), str(uuid.uuid4()) + '-final-' + os.path.basename(clip_path))
        file_merg = open(file_merg_path, "a")
        for i in range(times):
            file_merg.write("file '%s'\n" % tmp_clip_path)
        file_merg.close()
        cmd = "ffmpeg -y -f concat -safe 0 -i \"%s\" -codec copy \"%s\"" % (file_merg_path, final_clip_path)
        os.system(cmd)
        os.remove(tmp_clip_path)
        os.remove(file_merg_path)
        clip = VideoFileClip(final_clip_path)
        clip_duration = clip.duration
        clip.close()
        if clip_duration < loop_duration:
            return None
        return final_clip_path
    except:
        return None



import math
import random
class Summon:
    def __init__(self, direction, locx, locy, speedx, speedy, opacity, delay,s_w=100,s_h=100,m_w=1920,m_h=1080):
        self.direction = direction
        self.locx = locx
        self.locy = locy
        self.speedx = speedx
        self.speedy = speedy
        self.opacity = opacity #1000 is max
        self.delay = delay
        self.s_w = s_w
        self.s_h = s_h
        self.m_w = m_w
        self.m_h = m_h

    @staticmethod
    def setup(arr_direction, rang_locx, rang_locy, rang_speedx=[20,200], rang_speedy=[20, 200], rang_opacity=[0,800], rang_delay=[0,15], s_w=100, s_h=100, m_w=1920, m_h=1080):
        direction = random.choice(arr_direction)
        locx = random.randint(*rang_locx)
        locy = random.randint(*rang_locy)
        speedx = random.randint(*rang_speedx)
        speedy = random.randint(*rang_speedy)
        opacity = random.randint(*rang_opacity)/1000.0
        delay = random.uniform(*rang_delay)
        return  Summon(direction,locx,locy,speedx,speedy,opacity, delay,s_w,s_h,m_w,m_h)

    def to_middle_left(self, t):
        y = self.locy + self.speedy * math.sin(t)
        x = self.locx - self.speedx * t * 2
        return x, y

    def to_middle_right(self, t):
        y = self.locy + self.speedy * math.sin(t)
        x = self.locx + self.speedx * t * 2
        return x, y

    def to_top_left(self, t):
        y = self.locy - self.speedy * (1.5 * t + math.sin(2 * t))
        x = self.locx - self.speedx * t * 2
        return x, y

    def to_top_center(self, t):
        y = self.locy - self.speedy * (1.5 * t)
        # x=self.locx+random.choice([-1,1])*self.speedx*math.sin(t) #nhap nhay
        x = self.locx - self.speedx * math.sin(t)
        return x, y

    def to_top_right(self, t):
        y = self.locy - self.speedy * (1.5 * t + math.sin(2 * t))
        x = self.locx + self.speedx * t * 2
        return x, y

    def to_buttom_left(self, t):
        y = self.locy + self.speedy * (1.5 * t + math.sin(2 * t))
        x = self.locx - self.speedx * t * 2
        return x, y

    def to_buttom_center(self, t):
        y = self.locy + self.speedy * (1.5 * t)
        x = self.locx + self.speedx * math.sin(t)
        return x, y

    def to_buttom_right(self, t):
        y = self.locy + self.speedy * (1.5 * t + math.sin(2 * t))
        x = self.locx + self.speedx * t * 2
        return x, y

    def move(self, t):
        if t - self.delay < 0:
            return None, None
        t = t - self.delay
        if self.direction == 0:
            (x, y) = self.to_top_center(t)
        if self.direction == 1:
            (x, y) = self.to_top_right(t)
        if self.direction == 2:
            (x, y) = self.to_middle_right(t)
        if self.direction == 3:
            (x, y) = self.to_buttom_right(t)
        if self.direction == 4:
            (x, y) = self.to_buttom_center(t)
        if self.direction == 5:
            (x, y) = self.to_buttom_left(t)
        if self.direction == 6:
            (x, y) = self.to_middle_left(t)
        if self.direction == 7:
            (x, y) = self.to_top_left(t)
        if x > self.m_w or y > self.m_h or x+ self.s_w  < 0 or  y + self.s_h < 0:
            #print(x, y)
            return None, None
        return x, y




from bgeditor.dao.Summon import Summon
import random
from PIL import Image
from bgeditor.common.image_helper import ReduceOpacity
import numpy as np
from moviepy.editor import *
from bgeditor.common.utils import change_color_alpha
class Matrix:
    def __init__(self, bg_clip, rate_summon, arr_summon_tempalte, arr_direction, rang_locx, rang_locy,
                 rang_speedx=[20,200], rang_speedy=[20, 200], rang_opacity=[20,800], rang_size=[1,100],
                 arr_color=[], delay=3, m_w=1920, m_h=1080):
        """
        :param bg_clip:
        :param rate_summon:
        :param arr_summon_tempalte:
        :param duration: time of audio
        :param arr_direction:0,1,2,3,4,5,6,7 :12h,1h,3h,4h,6h,8h,10h
        :param rang_locx: [x1,x2], randge of location x asis
        :param rang_locy: [y1,y2], randge of location y asis
        :param rang_speedx:[v1,v2], speed of x asis
        :param rang_speedy:[v1,v2], speed of y asis
        :param rang_opacity:[0,1000], range of opacity
        :param rang_delay:[0,duration], time appear of star
        :param m_w:1920
        :param m_h:1080
        """
        self.rate_summon= rate_summon
        self.arr_summon_tempalte_path = arr_summon_tempalte
        self.arr_summon_tempalte_image = []
        self.m_w = m_w
        self.m_h = m_h
        self.arr_direction= arr_direction
        self.rang_locx=rang_locx
        self.rang_locy=rang_locy
        self.rang_speedx = rang_speedx
        self.rang_speedy= rang_speedy
        self.rang_opacity=rang_opacity
        self.arr_color=arr_color
        self.delay = delay
        self.rang_size= rang_size
        self.bg_clip=bg_clip
        self.t_summon= self.delay
    def setup_summon_template(self):
        for summon_template_path in self.arr_summon_tempalte_path:
            template_a = Image.open(summon_template_path).convert('RGBA')
            if len(self.arr_color) > 0:
                for color in self.arr_color:
                    img_layer = Image.fromarray(
                        change_color_alpha(np.asarray(template_a), color))
                    self.arr_summon_tempalte_image.append(img_layer)
            else:
                self.arr_summon_tempalte_image.append(template_a)
    def setup(self):
        try:
            self.list_summon = []
            self.setup_summon_template()
            return True
        except:
            pass
        return False
    def add_sumon(self,t):
        if t>= self.t_summon and t < self.t_summon+1:
            for i in range(self.rate_summon):
                summon = {}
                summon['template'] = random.choice(self.arr_summon_tempalte_image)
                s_h = s_w = random.randint(self.rang_size[0],self.rang_size[1])
                summon['skill'] = summon['template'].resize((s_w, s_h), Image.ANTIALIAS)
                summon['algorithm'] = Summon.setup(self.arr_direction, self.rang_locx, self.rang_locy, self.rang_speedx,
                                                   self.rang_speedy, self.rang_opacity, [self.t_summon, self.t_summon+1], s_w, s_h, self.m_w, self.m_h)
                summon['skill']= self.style_summon(summon)
                self.list_summon.append(summon)
            self.t_summon+=1

    def style_summon(self,summon):
        img_layer = summon['skill']
        return ReduceOpacity(img_layer, summon['algorithm'].opacity)

    def make_frame(self, t):
        img_tmp = Image.fromarray(self.bg_clip.get_frame(t))
        self.add_sumon(t)
        for summon in self.list_summon:
            x, y = summon['algorithm'].move(t)
            if x and y:
                img_rd = summon['skill']
                img_tmp.paste(img_rd, (int(x), int(y)), img_rd)
        return np.asarray(img_tmp)
    def make(self):
        return VideoClip(self.make_frame, duration = 3600)









import setuptools

setuptools.setup(
    name='coolbg',
    version='3.89',
    author="Thanh Hoa",
    author_email="thanhhoakhmt1@gmail.com",
    description="A Des of coolbg",
    long_description="Des",
    long_description_content_type="text/markdown",
    url="https://github.com/vtandroid/dokr",
    packages=setuptools.find_packages(),
    py_modules=['bgeditor'],
    install_requires=[
        'requests', 'numpy', 'moviepy','Pillow', 'youtube_dl','gbak'
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
 )